import numpy as np
import soundfile as sf
from scipy.signal import stft
from scipy.signal import istft
import matplotlib.pyplot as plt

from filter_signal import filterSignal
from bw_ext_upsampling_empty import bwExtUpsamplingEmpty

# Read in audiofiles
pfSpeechOrig, fSamplerateBroad = sf.read('speech_female_16000.wav')

# Define parameters (you can play with these)
iNfft             =  256
iFrameshift       =   64
fLowCutoffFreq    =  300
fHighCutoffFreq   = 3400    # Note: Needs to be smaller than 4000

# Define other parameters
fSamplerateNarrow = fSamplerateBroad / 2.0

# STFT of original signal
pfFreqVecBroad, pfFrameVecBroad, ppcSpeechOrig = stft(pfSpeechOrig, fs=fSamplerateBroad, nperseg=iNfft, noverlap=iNfft-iFrameshift)


# Filter signal (transmission channel)
pfFilteredSignal = filterSignal(pfSpeechOrig, fLowCutoffFreq, fHighCutoffFreq, fSamplerateBroad)
pfFilteredSignal = pfFilteredSignal[::2]
# STFT of filtered signal
pfFreqVecNarrow, pfFrameVecNarrow, ppcFilteredSignal = stft(pfFilteredSignal, fs=fSamplerateNarrow, nperseg=iNfft, noverlap=iNfft-iFrameshift)


# Upsampling with imaging approach
pfBwExtUpsampling, pfFilter = bwExtUpsamplingEmpty(pfFilteredSignal, fSamplerateBroad, iNfft, iFrameshift)
# STFT of upsampling with imaging approach
pfFreqVecBroad, pfFrameVecBroad, ppcBwExtUpsampling = stft(pfBwExtUpsampling, fs=fSamplerateBroad, nperseg=iNfft, noverlap=iNfft-iFrameshift)


# Write audio files
sf.write('speech_female_8000_narrow.wav', pfFilteredSignal, int(fSamplerateNarrow))
sf.write('speech_female_16000_bw_ext_upsampling.wav', pfBwExtUpsampling, fSamplerateBroad)


# Plot spectrogram of original signal
plt.figure(1)
plt.pcolormesh(pfFrameVecBroad, pfFreqVecBroad, 20*np.log10(np.abs(ppcSpeechOrig)), cmap='jet', vmin=-120, vmax=-20)
colorbar = plt.colorbar()
colorbar.set_label('Amplitude [dB]')
plt.title('Spectrogram of original signal')
plt.xlabel('Time [s]')
plt.ylabel('Frequency [Hz]')

# Plot spectrogram of filtered signal
plt.figure(2)
plt.pcolormesh(pfFrameVecNarrow, pfFreqVecNarrow, 20*np.log10(np.abs(ppcFilteredSignal)), cmap='jet', vmin=-120, vmax=-20)
colorbar = plt.colorbar()
colorbar.set_label('Amplitude [dB]')
plt.title('Spectrogram of filtered (narrowband) signal')
plt.xlabel('Time [s]')
plt.ylabel('Frequency [Hz]')

# Plot spectrogram of upsampling with imaging approach
plt.figure(3)
plt.pcolormesh(pfFrameVecBroad, pfFreqVecBroad, 20*np.log10(np.abs(ppcBwExtUpsampling)), cmap='jet', vmin=-120, vmax=-20)
colorbar = plt.colorbar()
colorbar.set_label('Amplitude [dB]')
plt.title('Spectrogram of bandwidth extended signal')
plt.xlabel('Time [s]')
plt.ylabel('Frequency [Hz]')

# Plot filter
plt.figure(4)
plt.plot(pfFreqVecBroad, pfFilter)
axes = plt.gca()
axes.set_ylim([0, 1.1])
plt.xlabel('Frequency [Hz]')
plt.ylabel('Filter amplitude')
plt.grid()

plt.show()